#include "GSP420NetworkProvider.h"


GSP420NetworkProvider::~GSP420NetworkProvider()
{
	Shutdown();
}

auto GSP420NetworkProvider::Init() -> void
{
	Network_Core::Instance()->Init();
}

auto GSP420NetworkProvider::Send(const char* address, int port, void* dataLocation, int dataSize, bool connectionless) -> int
{
	//Connectionless = UDP
	if(connectionless)
	{
		UDPSocket clientSocket(port);
		Packet dataToSend(dataLocation, dataSize);
		NetworkCore::Result returnValue = clientSocket.Send(address, port, dataToSend);
		return returnValue;
	}
	//Otherwise use TCP
	else
	{
		TCPSocket clientSocket(port);
		Packet dataToSend(dataLocation, dataSize);
		clientSocket.Connect(address, port);
		NetworkCore::Result returnValue = clientSocket.Send(dataToSend);
		clientSocket.Disconnect();
		return returnValue;
	}
}

auto GSP420NetworkProvider::Receive(int port, void* bufferLocation, int bufferSize, bool connectionless) -> int
{
	//Connectionless = UDP
	if(connectionless)
	{
		UDPSocket serverSocket(port);
		if(serverSocket.BindSocket() == NetworkCore::BIND_FAILED)
			return NetworkCore::BIND_FAILED;
		Packet dataToReceive(bufferLocation, bufferSize);
		NetworkCore::Result returnValue = serverSocket.Receive(dataToReceive, 0);
		return returnValue;
	}
	//Otherwise use TCP
	else
	{
		TCPListener serverSocket(port);
		Packet dataToReceive(bufferLocation, bufferSize);
		if(serverSocket.Listen())
		{
			NetworkCore::Result returnValue = serverSocket.Receive(dataToReceive);
			return returnValue;
		}
		return NetworkCore::RECEIVE_FAILED;
	}
}

auto GSP420NetworkProvider::Shutdown() -> void
{
	Network_Core::Instance()->Shutdown();
}