/*      TCPSocket Class for GSP-420 Network Core
		Team G: Alex Cary, Tylor Emmett, Javier Heard

		Changlog:
		1/17/15 - Tylor - Initial Version

		1/21/15 - Heard
		NOTES: Filled in class members, added notes on other classes that need to be further developed

		1/28/15 - Tylor - Modified methods to get winsock type data from the core types we've created.
*/

#include "TCPSocket.h"
#include "Network_Core.h"
#include <cassert>
#include <sstream>

TCPSocket::TCPSocket(int port):
	portNumber(port)
{
	CreateSocket();
}

TCPSocket::~TCPSocket()
{
	closesocket(gSocket);
}

bool TCPSocket::CreateSocket()
{
	if((gSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET)
	{
		Network_Core::Instance()->ShowLastError("TCPSocket::CreateSocket");
		return false;
	}
	return true;
}

NetworkCore::Result TCPSocket::Connect(const char* target, int port)
{
	//Fill in WinSock address structs
	addrinfo *address;
	//Convert port number to a string for getaddrinfo
	std::stringstream portString;
	portString << port;

	//Create addrinfo struct
	int getAddrResult = getaddrinfo(target, portString.str().c_str(), NULL, &address);
	
	if(getAddrResult != 0)
	{
		freeaddrinfo(address);
		Network_Core::Instance()->ShowLastError("TCPSocket::Connect - GetAddrResult");
		return NetworkCore::CONNECTION_FAILED;
	}
	
	//Connect to target address and port
	if(connect(gSocket, address->ai_addr, (int)address->ai_addrlen) == SOCKET_ERROR)
	{
		Network_Core::Instance()->ShowLastError("TCPSocket::Connect");
		closesocket(gSocket);
		freeaddrinfo(address);
		return NetworkCore::CONNECTION_FAILED;
	}
	freeaddrinfo(address);
	return NetworkCore::CONNECTED;
}

NetworkCore::Result TCPSocket::Send(Packet dataIn)
{
	if(send(gSocket, (char*)dataIn.GetLocation(), dataIn.GetSize(), NULL) == SOCKET_ERROR)
	{
		Network_Core::Instance()->ShowLastError("TCPSocket::Send");
		closesocket(gSocket);
		return NetworkCore::SEND_FAILED;
	}
	return NetworkCore::PACKET_SENT;
}

NetworkCore::Result TCPSocket::Receive(Packet dataOut)
{
	int result;

	//Attempt to get data
	result = recv(gSocket, (char*)dataOut.GetLocation(), dataOut.GetSize(), NULL);

	//Return result code.
	//Successfully received data
	if(result > 0)
	{
		return NetworkCore::PACKET_RECEIVED;
	}
	//Connection closed on other end
	else if(result == 0)
	{
		return NetworkCore::DISCONNECTED;
	}
	//Failed to receive data
	else
	{
		Network_Core::Instance()->ShowLastError("TCPSocket::Receive");
		return NetworkCore::RECEIVE_FAILED;
	}
}

NetworkCore::Result TCPSocket::Disconnect()
{
	//Shutdown the socket.
	if(shutdown(gSocket, SD_SEND) == SOCKET_ERROR)
	{
		Network_Core::Instance()->ShowLastError("TCPSocket::Disconnect");
		closesocket(gSocket);
		return NetworkCore::DISCONNECT_FAILED;
	}
	return NetworkCore::DISCONNECTED;
}

NetworkCore::Result TCPSocket::BindSocket()
{
	//Prepare socket address stucture.
	struct sockaddr_in server;
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = INADDR_ANY;
	server.sin_port = htons(portNumber);

	//Bind socket
	if(bind(gSocket, (struct sockaddr*)&server, sizeof(server)) == SOCKET_ERROR)
	{
		Network_Core::Instance()->ShowLastError("TCPSocket::BindSocket");
		return NetworkCore::BIND_FAILED;
	}
	return NetworkCore::SOCKET_BOUND;
}