//Simple application to test the funcionality of the Network Core. Can be set as a server or client, with either TCP or UDP protocols. Must be run as a server, then again as a client.

#include "INetworkProvider.h"
#include "GSP420NetworkProvider.h"

#include <string>
#include <iostream>

GSP420NetworkProvider networkProvider;

void TCPClientTest(const char* serverAddress, int port)
{
	//Data to send
	int message = 20;
	networkProvider.Send(serverAddress, port, (void*)message, sizeof(message), false);
}

void TCPServerTest(int port)
{
	//Prepate data buffer
	char buffer[1024];
	networkProvider.Receive(port, (void*) &buffer, sizeof(buffer), false);
	std::cout << buffer;
}

void UDPClientTest(const char* serverAddress, int port)
{
	//Data to send
	int message = 20;
	networkProvider.Send(serverAddress, port, (void*)message, sizeof(message), true);
}

void UDPServerTest(int port)
{
	//Prepate data buffer
	char buffer[1024];
	networkProvider.Receive(port, (void*) &buffer, sizeof(buffer), true);
	std::cout << buffer;
}

int main()
{
	
	
	networkProvider.Init();
		
	//Prompt for a test mode.
	int selection = 0;

	while((selection < 1) || (selection > 4))
	{
		std::cout << "Run test application as:" << std::endl
				  << "1. TCP Client" << std::endl
				  << "2. TCP Server" << std::endl
				  << "3. UDP Client" << std::endl
				  << "4. UDP Server" << std::endl;
	
		std::cin >> selection;
		if((selection < 1) || (selection > 4))
		{
			std::cout << "Please enter a valid selection" << std::endl;
		}
	}
	
	//Execute selected mode.
	int port;
	std::string buffer;
	switch(selection)
	{
	case 1:
		std::cout << "Enter port number: " << std::endl;
		std::cin >> port;
		TCPClientTest("127.0.0.1", port);
		break;
	case 2:
		std::cout << "Enter port number: " << std::endl;
		std::cin >> port;
		TCPServerTest(port);
		break;
	case 3:
		std::cout << "Enter port number: " << std::endl;
		std::cin >> port;
		UDPClientTest("127.0.0.1", port);
		break;
	case 4:
		std::cout << "Enter port number: " << std::endl;
		std::cin >> port;
		UDPServerTest(port);
		break;
	}

	networkProvider.Shutdown();
	system("pause");
}

