/*      UDPSocket Class for GSP-420 Network Core
		Team G: Alex Cary, Tylor Emmett, Javier Heard

		Changlog:
		1/17/15 - Tylor - Initial Version
		1/31/15 - Tylor - Added TCP Code.



*/

#include "UDPSocket.h"
#include "Network_Core.h"
#include <cassert>

UDPSocket::UDPSocket(int port): portNumber(port)
{
	CreateSocket();
}

UDPSocket::~UDPSocket()
{
	closesocket(gSocket);
}

bool UDPSocket::CreateSocket()
{
	//Create socket.
	if((gSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == INVALID_SOCKET)
	{
		Network_Core::Instance()->ShowLastError("UDPSocket::CreateSocket");
		return false;
	}
	

	return true;
}

NetworkCore::Result UDPSocket::BindSocket()
{
	//Prepare socket address stucture.
	struct sockaddr_in server;
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = INADDR_ANY;
	server.sin_port = htons(portNumber);

	//Bind socket
	if(bind(gSocket, (struct sockaddr*)&server, sizeof(server)) == SOCKET_ERROR)
	{
		Network_Core::Instance()->ShowLastError("UDPSocket::BindSocket");
		return NetworkCore::BIND_FAILED;
	}
	return NetworkCore::SOCKET_BOUND;
}

NetworkCore::Result UDPSocket::Send(const char* target, int port, Packet dataIn)
{
	//Setup server address data.
	struct sockaddr_in other;
	other.sin_addr.S_un.S_addr = inet_addr(target);
	other.sin_port = htons(portNumber);
	other.sin_family = AF_INET;
	int otherLen = sizeof(other);
	
	//Send packet
	if(sendto(gSocket, (char*)dataIn.GetLocation(), dataIn.GetSize(), 0, (struct sockaddr*) &other, otherLen) == SOCKET_ERROR)
	{
		Network_Core::Instance()->ShowLastError("UDPSocket::Send");
		return NetworkCore::SEND_FAILED;
	}

	return NetworkCore::PACKET_SENT;
}

NetworkCore::Result UDPSocket::Receive(Packet dataOut, int sizeReceived)
{
	struct sockaddr_in other;

	//Clear packet data.
	memset(dataOut.GetLocation(), '\0', dataOut.GetSize());

	//Get size of other address struct.
	int otherLen = sizeof(other);

	//Receieve data. This is a blocking call.
	if(sizeReceived = recvfrom(gSocket, (char*)dataOut.GetLocation(), dataOut.GetSize(), 0, (struct sockaddr*)& other, &otherLen) == SOCKET_ERROR) 
	{
		return NetworkCore::RECEIVE_FAILED;
	}

	return NetworkCore::PACKET_RECEIVED;
}